/*================================================================================*/
/* Filename:   power trend grouped counts analytical.sas                          */
/* Version:    09 Sep 2006                                                        */
/* Purpose:    Calculate approximate power to detect trends across grouped count  */
/*             data.  See Gray and Burlew (submitted) for details.                */
/* Input data: see below                                                          */
/* Outputs:    Precision (full confidence interval) on log scale. Note possible   */
/*             minor dependence of precision on hypothesized trend when not using */
/*             constant CV.                                                       */
/*             Power to detect trends over multiple, independent sampling events  */
/*             [associated null hypothesis (likely naive), is trend = 0].         */
/*             "meanatmidpoint" = count mean at midpoint of sampling period--     */
/*             given alternative trend hypothesis.                                */
/*             CV, if not entered as constant, calculated at "meanmidpoint."      */
/*             % sampling variance on log scale.                                  */
/* Caveats:    1. 1-sided tests assume alternate hypothesis in same direction     */
/*             as effect.                                                         */ 
/* Author:     Brian Gray, USGS                                                   */
/* Disclaimer: This material is provided "as is" by the author. There are no      */
/*             warranties, expressed or implied, as tofitness for a               */
/*             particular purpose regarding the materials or code contained       */
/*             herein. The author is not responsible for errors in this           */
/*             material as it now exists or will exist, nor does the author       */
/*             provide technical support for it. While the author is not          */
/*             responsible for errors in this material, he will appreciate being  */
/*             notified of those errors.  Please contact Brian Gray at            */
/*             brgray@usgs.gov with these comments.                               */
/*================================================================================*/


/****************** PROVIDE INPUTS *********************/
%let title = species X count index;
%let alpha = 0.10;                * set Type I error rate;
%let startmeans = 1,10;           * enter count mean(s) at time=0.  note that these = exp(intercept + tau2events/2).
                                    if CV > 0, enter single startmean to eliminate redundant output;
%let trendspct = 1,5;             * enter hypothesized percent change in the mean for a unit change in time. this
                                    number = 100*(exp(b1) - 1), where exp(b1) = the trend multiplier and b1 = trend
                                    coefficient on log scale;
%let NBdisp = 0,2;                * enter NB dispersion parameters.  enter 0 for Poisson assumption;
%let NBvar = 2;                   * enter 1 for NB1 or 2 for NB2 distributional assumptions, and either 
                                    for Poisson assumptions or when entering positive CV value;
%let CV = 0;                      * enter positive coefficient of variation (CV) to circumvent parametric sampling variance
                                    assumptions.  Else enter 0;
%let events = 5,10,30;            * enter # sampling events (>= 3) over which power to be estimated;
%let interval = 1;                * enter interval, in years, between events (e.g., 1 for annual, 2 for biennial);
%let n = 5,10,50;                 * enter sample size(s) per site and sampling event;
%let tau2events = .05,.1;         * enter variance(s) among sampling events (estimated on log scale);
%let tails = 2;                   * define whether one (1)- or two (2)-tailed test. Direction of trend presumed that of trend under alternate hypothesis;


/****************** ESTIMATE POWER UNDER ABOVE ASSUMPTIONS ***********************/
data powert;
 do startmean = &startmeans;
   if &CV then startmean = .;
 do NBdisp = &NBdisp;
 do trendpct = &trendspct;
   trendmult = 1 + trendpct/100;  * trend multiplier, with reference value = 1;
 do n = &n;
 do tau2events = &tau2events;
 do events = &events;

   * estimate slope ('trend') on log scale under fixed trend assumption;
   beta1 = log(trendmult);

   * calculate mean at half sampling period;
   if &CV then meanmidpoint = .;	
     else meanmidpoint = startmean * trendmult**(((events-1)*&interval)/2);

   * calculate negative binomial/Poisson sampling variance;	
   if &NBvar in (1,2) and &CV le 0 then sigma2 = meanmidpoint + NBdisp*(meanmidpoint**&NBvar); else sigma2 = .;

   * calculate conditional sampling variance of mean by delta method;
   if sigma2 ne . then sigma2logmean = sigma2/(n*(meanmidpoint**2));
     else if &CV gt 0 then sigma2logmean = (&CV**2)/n; else sigma2logmean = .;

   * set NB dispersion parameter to zero if CV > 0;
   if &CV gt 0 then NBdisp = .;

   * calculate approximate CV where CV not presumed constant;
   if &CV = 0 then CV = round(sqrt(sigma2)/meanmidpoint,.01); else CV = &CV;

   * calculate approximate variance of the slope;
   stt = events*(&interval**2)*(-1+events**2)/12;  * derived from discrete uniform distribution, or as sum of 1st k squared integers (Standard Mathematical Tables, p54);
   varlogmean = tau2events + sigma2logmean;        * by eqn 3.20 Raudenbush and Bryk 2002 and by inference from eqn 2.9 Myers 1990;
   var_b1 = varlogmean/stt;                        * eqn 2.9 Myers 1990 & eqn 3.3b Neter et al. 1985;
   SE_b1 = sqrt(var_b1);

   * estimate power;
   df = events-2;
   tcrit = tinv(1-&alpha/&tails,df);
   delta = beta1/SE_b1;                                         * eqn 3.26 Neter et al 1985;
   if trendpct lt 0 then power1t = probt(-tcrit,df,delta);      * prob of noncentral t(df) lt tcrit--under assumption of negative trend;
     else power1t = 1-probt(tcrit,df,delta);                    * prob of noncentral t(df) gt tcrit--under assumption of nonnegative trend;
   power2t = probt(-tcrit,df,delta)+(1-probt(tcrit,df,delta));  * prob of noncentral t(df) lt or gt |tcrit| under 2-tailed assumption;
   if &tails = 1 then power = power1t; else power = power2t;

   * calculate *expected* confidence intervals for hypothesized trend. confidence intervals use central T statistics because the 
     approach does not presume an alternative hypothesis;
   lcl = trendmult * exp(-tinv(1-&alpha/&tails,df)*SE_b1);
   ucl = trendmult * exp(tinv(1-&alpha/&tails,df)*SE_b1);
   if &tails = 1 and trendpct < 0 then ucl = .; if &tails = 1 and trendpct > 0 then lcl = .;

   * calculate precision as confidence interval on trend parameter, gamma01;
   precision = 2*tinv(1-&alpha/&tails,df)*SE_b1;

   * output;
   if df ge 1 then output;

 end;
 end;
 end;
 end;
 end;
 end;
run;



/************************** PRINT POWER APPROXIMATIONS ************************/
options pagesize=40 ls=165;
proc print data=powert noobs;
 format meanmidpoint power 6.2;
 title1 height=2 justify=center font=swissb "Precision and power approximations for &title., alpha = &alpha, tails = &tails..";
 title2 height=2 justify=center font=swissb "Precision = 1/2 confidence interval (CI) on trend parameter [log(1+trendpct/100)]. CIs on trend multiplier ['trendmult', 1 + trendpct/100].";
 var startmean meanmidpoint trendpct NBdisp CV tau2events events n df SE_b1 precision power;
run;
