/*----------------------------------------------------------------------------*/
/*  Program Name: SIMULATE POWER TREND 2 LEVEL NB OUTCOMES                    */
/*  SAS Version:  9.1 Windows                                                 */
/*  Purpose:      Simulate power to detect trends across groups of negative   */
/*                binomially-distributed random variates, with normally       */
/*                distributed group effects. See Gray and Burlew (submitted)  */
/*                for additional details.                                     */
/*  Notes:        Intercept variance estimates are biased low due to the      */
/*                bias associated with maximum likelihood estimation.         */
/*                                                                            */
/*  Authors:      Michele M. Burlew, Episystems Inc and Brian Gray, USGS      */
/*  Date:         July-September 2005                                         */
/*  Disclaimer:   This material is provided "as is" by the authors.  There    */
/*                are no warranties, expressed or implied, as to fitness for a*/
/*                particular purpose regarding the materials or code contained*/
/*                herein. The authors are not responsible for                 */
/*                errors in this material as it now exists or will exist, nor */
/*                do the authors provide technical support for it. While the  */
/*                authors are not responsible for errors in this material,    */
/*                they will appreciate being notified of those errors. Please */
/*                contact Brian Gray at brgray@usgs.gov or Michele Burlew     */
/*                at mmburlew@comcast.net with these comments.                */
/*  Last update:  29 Aug 2006 14:20:20                                        */
/*----------------------------------------------------------------------------*/

%macro trendpowercount(outcometitle=,startmean=,
                       NBdisp=, tau2=, trend=, n=,
                       groups=, smplintval=, maxiter=, alpha=.1,tail=2,
                       randomseed=);

  %*****************************************************************************;
  %*                                                                            ;
  %* This macro program is defined with keyword parameters as specified on the  ;
  %* MACRO definition statement. When writing a call to this program, always    ;
  %* specify the parameter name, an equal sign, and the value to assign to the  ;
  %* parameter.                                                                 ;
  %*                                                                            ;
  %* PARAMETER DEFINITIONS:                                                     ;
  %* OUTCOMETITLE= text of title to place in PROC REPORT report                 ;
  %*   CAUTION: Title text cannot contain punctuation                           ;
  %*                                                                            ;
  %* STARTMEAN= starting mean (i.e. the equivalent of exp(intercept+tau00/2))   ;
  %* NBDISP   = negative binomial dispersion parameter                          ;
  %* TAU2     = variance among group (event) means on log scale (zero accepted)	;
  %* TREND    = r, change per time or distance unit                             ;
  %* N        = within-group (event) sample size                                ;
  %* GROUPS   = number of sampling events spaced one or more years apart        ;
  %*               (years if samples are nested within years)                   ;
  %* SMPLINTVAL = sample interval (eg every year (1), every 2 years (2)]        ;
  %* MAXITER  = number of simulated data sets this macro should generate        ;
  %* ALPHA = alpha level for analysis. Value as entered into NLMIXED            ;
  %*   is doubled if parameter TAIL=1                                           ;
  %* TAIL = 1 or 2. Value of TAIL determines if alpha value entered             ;
  %*   into NLMIXED is doubled                                                  ;
  %* RANDOMSEED = 0 or positive integer. Specify 0 when you want the stream of	;
  %*   random numbers to be initialized by the computer clock and the results 	;
  %*   are not replicable. Specify a positive integer when you want replicable	;
  %*   results. SAS generates the same series of numbers when the seed is a     ;
  %*   positive integer.                                                        ;
  %*                                                                            ;
  %* Notes:                                                                     ;
  %*                                                                            ;
  %* 1.	Convergence status and the presence of any gradient gt 0.001 retained.	;
  %*    These settings may be modified by searching for "GRADIENT."  Models     ;
  %*    with either error excluded from power estimation.                       ;
  %* 2. For large numbers of iterations, notes to the log may fill the log      ;
  %*    window. In this case, options nonotes may be used with caution. Search  ;
  %*    for "nonotes" to find the appropriate line to unremark.                 ;
  %* 3. Upper and lower confidence limits (CL) on trend are estimated using both;
  %*    parametric and nonparametric approaches. The former estimates the CL    ;
  %*    using a T-based interval, is appended by an 'M' (for 'model', i.e.      ;
  %*    'LCL_b1m' and 'UCL_b1m') and are appropriate only when within-cluster   ;
  %*    sample sizes are large (e.g. nj>30) or when group-scale variance is     ;
  %*    large relative to sample-scale variance. The nonparametric estimates are;
  %*    rank-based, are appropriate for small within-cluster sample sizes but   ;
  %*    require more iterations-particularly at extreme alpha levelsl All CLs   ;
  %*    first esimated on the link scale, and then exponentiated.               ;
  %*****************************************************************************;

  %* RETAIN CLOCK TIME WHEN MACRO INVOKED;
  %let strttime=%sysfunc(datetime());

  /* DELETE OLD TEMPORARY DATA FILES */
  proc datasets library=work memtype=data;
    delete generate param_ests estvar pctls
           estpower nbdisp_hat b0_hat convstat pwr
           temppower temp_param temp_param2
           dimensions;
  run;
  quit;


  %* ASSIGN NAME TO DATA SET CONTAINING GENERATED OBS;
  %let dataset=generate;


  %****************************************;
  %* LIMITED PARAMETER CHECKING;
  %****************************************;

  %* CHECK ALPHA PARAMETER;
  %if &alpha= or (%datatyp(&alpha) ne NUMERIC) or %sysevalf(&alpha le 0) or
         %sysevalf(&alpha ge 1) %then %do;
    %put **********************************************************************;
    %put **********************************************************************;
    %put * ALPHA parameter must be specified and must be between 0 and 1;
    %put * Value specified for ALPHA=&alpha;
    %put * Please respecify ALPHA parameter and rerun program;
    %put **********************************************************************;
    %put **********************************************************************;
    %goto exit;
  %end;

  %* CHECK TAIL PARAMETER;
  %if &tail= or (&tail ne 1 and &tail ne 2) %then %do;
    %put **********************************************************************;
    %put **********************************************************************;
    %put * TAIL parameter must be specified and must be 1 or 2;
    %put * Value specified for TAIL=&tail;
    %put * Please respecify TAIL parameter and rerun program;
    %put **********************************************************************;
    %put **********************************************************************;
    %goto exit;
  %end;

  %* ADJUST ALPHA IF ONE TAILED PROCESSING;
  %let alphaproc=&alpha;
  %if &tail=1 %then %let alphaproc=%sysevalf(&alpha*2);

  %* SET PCTLPTS FOR PROC UNIVARIATE;
  %if &tail=2 %then %do;
    %let pctlptl=%sysevalf(100*&alpha/2);
    %let pctlptu=%sysevalf(100*(1-&alpha/2));
  %end;
  %else %do;
    %if %sysevalf(&trend lt 0) %then %do;
      %let pctlptl=%sysevalf(100*&alpha);
      %let pctlptu=100;
    %end;
    %else %if %sysevalf(&trend gt 0) %then %do;
      %let pctlptl=0;
      %let pctlptu=%sysevalf(100*(1-&alpha));
    %end;
  %end;


  %* CHECK RANDOMSEED PARAMETER;
  %if &randomseed= or (%datatyp(&randomseed) ne NUMERIC) or
     %index(&randomseed,.) gt 0 or %index(&randomseed,-) gt 0 %then %do;
    %put **********************************************************************;
    %put **********************************************************************;
    %put * RANDOMSEED parameter must be specified and must be 0 or a positive integer;
    %put * Value specified for RANDOMSEED=&randomseed;
    %put * Please respecify RANDOMSEED parameter and rerun program;
    %put **********************************************************************;
    %put **********************************************************************;
    %goto exit;
  %end;



  %* DETERMINE GROUPS LOOP;
  %*   WHEN ONE VALUE SPECIFIED, ONLY PROCESS THAT SPECIFIC GROUP;
  %*   WHEN LIST OF VALUES SPECIFIED, INSERT APPROPRIATE COMMAS FOR INSERTION;
  %*     LATER IN DO STATEMENT;
  %*   OTHERWISE LEAVE GROUPS MACRO PARAMETER AS IS, BUT CHECK IF VALID;
  %*     COMPONENTS;

  %let groups=%qupcase(&groups);

  %* ASSUME LIST OF VALUES SPECIFIED FOR GROUPS;
  %if %scan(&groups,2) ne %str(TO) %then %do;
    %let groupstring=%sysfunc(compbl(&groups));
    %let groupelements=1;
    %let maxgroups=;
    %do %until(%scan(&groupstring,&groupelements,' ')=);
      %if %scan(&groupstring,&groupelements,' ') ne %then
          %let maxgroups=%scan(&groupstring,&groupelements,' ');
      %let groupelements=%eval(&groupelements+1);
    %end;
    %let groupelements=%eval(&groupelements-1);
  %end;
  %* ASSUME DO LOOP EXPRESSION SPECIFIED FOR GROUPS. EVALUATE FIRST 3 COMPONENTS;
  %* FIRST AND THIRD MUST BE A NUMBER, SECOND MUST BE TEXT TO;
  %else %if %datatyp(%scan(&groups,1))=NUMERIC and
        %qupcase(%scan(&groups,2)) eq %str(TO) and
        %datatyp(%scan(&groups,3))=NUMERIC %then %do;
    %let startval=%scan(&groups,1);
    %let endval=%scan(&groups,3);
    %let byval=%scan(&groups,5);
    %if &byval= %then %let byval=1;
    %let groupstring=;
    %let groupelements=0;
    %let maxgroups=&endval;
    %do g=&startval %to &endval %by &byval;
      %let groupstring=%trim(&groupstring) &g;
      %let groupelements=%eval(&groupelements+1);
    %end;
  %end;
  %else %do;
    %put **********************************************************************;
    %put **********************************************************************;
    %put * INVALID SPECIFICATION OF GROUPS PARAMETER: &groups;
    %put * GROUPS parameter may be specified one of three ways:;
    %put *   1. Single value to produce analysis of only one group (e.g. 3);
    %put *   2. List of values to produce an analysis for each value(e.g. 5,10,15,16);
    %put *   3. Written as DO statement expression (e.g. 5 TO 20 by 5);
    %put * Please respecify GROUPS parameter and rerun program;
    %put **********************************************************************;
    %put **********************************************************************;
    %goto exit;
  %end;




  %* GENERATE MAXITER NUMBER OF DATA SETS;
  %do iter=1 %to &maxiter;
    %generate_nbrv3

    %do g=1 %to &groupelements;
      %let period=%scan(&groupstring,&g);

      %nlmixed_nb3;
    %end;
  %end;


  %* CALCULATE CLOCK TIME TO GET TO THIS POINT;
  %let endtime=%sysfunc(datetime());
  %let seconds=%sysevalf(&endtime-&strttime);

  %put ;
  %put ;
  %put *********************************************;
  %put Start Time: %sysfunc(putn(&strttime,datetime19.));
  %put End Time:   %sysfunc(putn(&endtime,datetime19.));
  %put Duration:   %sysfunc(putn(&seconds,time9.));
  %put *********************************************;
  %put ;
  %put ;



  /* TURN RESULTS WINDOW OUTPUT ON */
  ods results;
  ods listing;

  %* CALL MACRO STATS TO ESTIMATE VARIANCE COMPONENT STATISTICS;
  %stats

%exit:
%mend trendpowercount;


%macro stats;
  %* MACRO STATS ESTIMATES VARIANCE COMPONENT STATISTICS AND IS INVOKED ONCE ;
  %* BY MACRO TRENDPOWERCOUNT ;

  /* COMPUTE FOLLOWING STATS ONLY WHERE 4 GRADIENTS ALL LESS THAN */
  /* CUTOFF OF ABS(.001) AND CONVERGENCE STATUS=0                 */
  proc sort data=param_ests(where=(gradltcutoff='Y' and convstatus=0)) out=pwr;
    by period iteration;
  run;


  %let param_logsig0=0;
  data pwr;
    set pwr;

    if parameter="logsig0" then do;
      call symput("PARAM_LOGSIG0",1);
      var=exp(2*estimate);
    end;
    else var=.;

    if lower lt 0 and upper gt 0 then statsig = 0 ;
    else statsig = 1;

    if &tail=1 and &trend lt 0 then if upper gt 0 then statsig = 0;
    else statsig = 1;
    if &tail=1 and &trend gt 0 then if lower lt 0 then statsig = 0;
    else statsig = 1;
  run;

  %if &param_logsig0=1 %then %do;
    proc means data=pwr(where=(parameter="logsig0")) noprint;
      by period;
      var var;
      output out=estvar(drop=_TYPE_ _FREQ_) mean=mean_tau2_est
                        median=median_tau2_est min=min_tau2_est
                        max=max_tau2_est n=iter_est;
     run;
  %end;
  %else %do;
    %* DEFINE EMPTY DATA SET WHEN LOGSIG0 PARAMETER IS MISSING;
    data estvar;
      length mean_tau2_est median_tau2_est min_tau2_est max_tau2_est
             iter_est period 8;
      if _n_ ge 1 then stop;
  %end;

  /* ESTIMATE POWER AND ALSO COMPUTE MEAN QUADRATURE POINTS */
  proc means data=pwr(where=(parameter="b1_hat")) noprint;
    id gradltcutoff convstatus;
    by period;
    var estimate statsig quadpoints standarderror;
    output out=estpower(drop=_TYPE_ _FREQ_)
             mean=mean_b1_est power_est mean_quadpts mean_SE_b1_est
             n=iter
             std=sd_b1_est;
  run;

  proc univariate data=pwr(where=(parameter="b1_hat")) noprint;
    id gradltcutoff convstatus;
    by period;
    var estimate;
    output out=pctls pctlpre=p pctlpts=50 &pctlptl &pctlptu;
      * output median and rank-based lower and upper CIs on the mean;
  run;
  proc sql noprint;
    select name into :varname0-:varname3
    from dictionary.columns
    where libname='WORK' and memname='PCTLS';
  quit;
  proc datasets library=work nolist;
    modify pctls;
    rename &varname1=median_b1_est
           &varname2=lcl &varname3=ucl;
  run;
  quit;

  proc means data=pwr(where=(parameter="b0_hat")) noprint;
    by period;
    var estimate standarderror;
    output out=b0_hat(drop=_TYPE_ _FREQ_)
             mean=mean_b0_est mean_SE_b0_est;
  run;
  proc means data=pwr(where=(parameter="NBdisp_hat")) noprint;
    by period;
    var estimate standarderror;
    output out=nbdisp_hat(drop=_TYPE_ _FREQ_)
             mean=mean_NBdisp_est mean_SE_NBdisp_est;
  run;

  data temppower;
    merge estvar estpower pctls b0_hat nbdisp_hat;
      by period;

    retain tau2 &tau2 startmean &startmean trend &trend NBdisp &NBdisp n &n
           maxgroups &maxgroups smplintval &smplintval alpha &alpha tail &tail
           randomseed &randomseed;

    /* For TAIL=1, set LCL/UCL to missing based on TREND */
    if &tail=1 then do;
      if &trend < 0 then ucl=.;
      else if &trend > 0 then lcl=.;
    end;

    format rundate datetime16.
           mean_NBdisp_est mean_SE_NBdisp_est mean_SE_b0_est mean_SE_b1_est
           mean_b0_est mean_b1_est mean_quadpts;

    rundate=datetime();
  run;

  proc append base=power data=temppower;
  run;

  options orientation=landscape ls=140;
  ods listing;

  title1 "Power to detect trend of &outcometitle";
  title2 'Model inputs, and estimated power';

  proc report data=power(where=(gradltcutoff='Y' and convstatus=0))
               nowd box split='_' colwidth=6 spacing=0;
    column rundate ('-Program Inputs-' startmean trend nbdisp tau2
                                       period iter n smplintval)
          ('-Model estimate statistics-'
           mean_b0_est mean_b1_est mean_SE_b1_est
           sd_b1_est median_b1_est lcl ucl
           mean_NBdisp_est mean_tau2_est median_tau2_est) power_est;

    define rundate / order noprint;
    define startmean / 'start_mean' width=5;
    define trend / width=5;
    define nbdisp / width=4 'NB_disp';
    define tau2 / width=5;
    define period / 'groups';
    define iter / 'Iter_ations';
    define n / display width=4;
    define smplintval / 'Inter_val' width=5;
    define mean_b0_est / format=6.3 'mean b0^';
    define mean_b1_est / format=6.3 'mean b1^';
    define mean_SE_b1_est / format=6.3 'mean SE b1^';
    define sd_b1_est / format=6.3 'SD b1^';
    define median_b1_est / format=6.3 'median b1^';
    define lcl / format=6.3 'LCL b1^';
    define ucl / format=6.3 'UCL b1^';
    define mean_NBdisp_est / format=6.2 'mean NBdisp^';
    define mean_tau2_est / format=6.3 'mean tau2^';
    define median_tau2_est / format=6.3 'median tau2^';
    define power_est / 'Power_(est)' format=6.2;


    compute before rundate;
      line @4 "Run Date and Time: " rundate datetime16.;
    endcomp;

  run;

  options orientation=portrait;

  title1;
%mend stats;

%macro generate_nbrv3;
  %* MACRO GENERATE_NBRV3 GENERATES OBSERVATIONS FOR ANALYSIS AND IS INVOKED ;
  %* MAXITER NUMBER OF TIMES ;

  data &dataset;
    retain u0 0;
    randomseed=&randomseed*&iter;

    do group=0 to &maxgroups-1;
      %if %sysevalf(&tau2 gt 0) %then %do;
        call rannor(randomseed,norvalue);
        u0=sqrt(&tau2)*norvalue;
      %end;
      do ii=1 to &n;
        shape=1/&NBdisp;

        /* TO ENSURE E[GAMMA RV] = 1 AND "INTERCEPT IDENTIFICATION," SET SCALE=1/SHAPE */
        scale=&NBdisp;
        call rangam(randomseed,shape,gamvalue);
        xgammaij=scale*gamvalue;

        /* SLOPE ON LOG SCALE */
        beta1=log(1+&trend);

        /* GENERATE MEAN OF NB VARIATE */
        mean=&startmean*exp(beta1*group*&smplintval + u0)*xgammaij;
           * upate median by trend, group, and observational effects;

        /* GENERATE RANDOM VARIATE */
        call ranpoi(randomseed,mean,y);
        yplus1=y+1;  * define variable=(y+1) to avoid calculation problems in NLMIXED;
        output;
      end;
    end;
  run;

%mend generate_nbrv3;

%macro nlmixed_nb3;
  %* MACRO NLMIXED_NB3 EXECUTES THE NLMIXED PROCEDURE ON THE SIMULATED DATA. ;
  %* THIS MACRO EXECUTES ONCE FOR EACH PERIOD AND ITERATION.;


  %* DETERMINE DF BASED ON TAU2 AND PERIOD;
  %if %sysevalf(&tau2 gt 0) %then %let df=%eval(&period-2);
  %else %let df=%eval(&period-1);


  ods output parameterestimates=temp_param
             convergencestatus=convstat
             dimensions=dimensions;

  proc nlmixed data=&dataset(where=(group lt &period))
               tech=trureg
               df=&df
               alpha=&alphaproc;

    /* PARAMETER ESTIMATES */
    parms b0_hat b1_hat = 0
          NBdisp_hat=1

          %if %sysevalf(&tau2 ne 0) %then %do;
            logsig0=0
          %end;
          ;

    /* LINK-LEVEL MEAN STATEMENTS */
    eta_lambda =  b0_hat + b1_hat*group*&smplintval
                  %if &tau2 ne 0 %then %do;
                    + u0_hat
                  %end;
                  ;
    lambda = exp(eta_lambda);

    /* LIKELIHOOD STRUCTURE */
    loglike = y*log(NBdisp_hat*lambda) - (y+1/NBdisp_hat)*log(1+NBdisp_hat*lambda) +
              lgamma(y+1/NBdisp_hat)-lgamma(1/NBdisp_hat); *-lgamma(yplus1); * add
              constant if comparing across distributional assumptions;

   /* RANDOM LINES - EXECUTE ONLY IF TAU2 NE 0 */
   %if %sysevalf(&tau2 ne 0) %then %do;
     /* INTERCEPT VARIANCE */
     v0 = exp(2*logsig0);
     random u0_hat ~ N(0,v0) subject = group;
   %end;

   /* FIT THE MODEL */
   model y ~ general(loglike);
  run;

  /* EVALUATE GRADIENTS */
  data _null_;
    set temp_param end=eof;

    retain gradltcutoff 'Y';

    if parameter in ('b0_hat','b1_hat','nbdisp_hat','logsig0') and
         abs(gradient) ge 1e-3 then gradltcutoff='N';

    if eof then call symput('GRADLTCUTOFF',gradltcutoff);
  run;


  /* CHECK IF QUADRATURE POINTS IN DIMENSIONS */
  /* INITIALIZE TO MISSING */
  %let quadpoints=.;
  data _null_;
    set dimensions(where=(descr="Quadrature Points"));

    call symput("QUADPOINTS",value);
  run;

  /* UPDATE PARAMETER DATASET */
  data temp_param2;
    set temp_param;

    if _n_=1 then set convstat(rename=(reason=convreason status=convstatus));


    retain period &period iteration &iter gradltcutoff "&gradltcutoff"
           quadpoints &quadpoints;
  run;

  proc append base=param_ests data=temp_param2;
  run;
%mend nlmixed_nb3;





/* INITIALIZE AND RESET OPTIONS */
title;
footnote;
goptions reset=all;
options pageno=1;

/* SET OPTIONS TO SAVE RESOURCES */
/* TURN OFF TRACKING OF OUTPUT THAT IS GENERATED BY ODS IN THE RESULTS WINDOW */
ods noresults;

/* CONTROL DISPLAY OF NOTES */
options notes;

/* USE THE FOLLOWING STATEMENT WHEN ITERATIONS */
/* HIGH AND POTENTIAL FOR LOG WINDOW TO FILL   */
/* WITH NOTES HIGH. WHEN USED, ALSO EXECUTE    */
/* OPTIONS NOTES STATEMENT IN THE RESTORE      */
/* OPTIONS SECTION AT THE END OF THE PROGRAM.  */
/* THIS WILL REINSTATE DISPLAY OF NOTES IN THE */
/* LOG WINDOW FOR SUBSEQUENT PROCESSING.       */
*options nonotes;


/* SUPPRESS SENDING OUTPUT TO THE LISTING DESTINATION */
ods listing close;

%trendpowercount(outcometitle=Species X abundance index,
                 startmean=1,  /* Enter starting mean (i.e. the equivalent of exp(intercept+tau2/2)) */
                 NBdisp=2,     /* Enter negative binomial dispersion parameter */
                 tau2=.25,    /* Enter group variance on log scale, zero(0) allowed */
                 trend=.05,    /* Enter q, change per time or distance unit              */
                 n=30,         /* Enter within-group sample size */
                 groups=5 to 20 by 5,  /* Enter number of sampling events spaced one or more years apart for
                                          which to estimate power (e.g. 5 10 15 20).
                                          Format=x to y by z (e.g. 10 to 20 by 5), or list (e.g. 10 12 15 17 20).
                                          Separate the numbers with spaces */
                 smplintval=1,  /* Enter sampling frequency (eg every year (1), every 2 years (2)] */
                 maxiter=2,    /* Enter number of iterations > 0 */
                 alpha=.1,     /* Enter alpha level */
                 tail=2,       /* Choose between 1 or 2 tailed tests */
                 randomseed=5) /* Choose between 0 for results that are not replicable and a positive
                                  integer for replicable results */

/* RESTORE OPTIONS */
ods listing;
ods results;

/* USE THE FOLLOWING STATEMENT WHEN THE OPTIONS  */
/* NONOTES STATEMENT ABOVE HAS EXECUTED. OPTIONS */
/* NOTES REINSTATES DISPLAY OF NOTES IN THE LOG  */
/* WINDOW FOR SUBSEQUENT PROCESSING.             */
*options notes;
